#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include "neo.h"

void neo_command_help(int argc, char **argv)
{
  FILE *pager;
  char *pstr;

  argc--;
  argv++;

  pstr=getenv("PAGER");
  /* if (!pstr) pstr=getenv("VISUAL"); */

  if (!pstr) {
    pager=stdout;
  } else {
    pager=popen(pstr, "w");
  }

  /*
   * MAIN HELP
   */
  if (argc==0) {
    fprintf(pager,"NEO HELP                                                         (main help)\n");
    fprintf(pager,"                                                                            \n");
    fprintf(pager,"Type 'help' followed by any of the following for specifics.                 \n");
    fprintf(pager,"                                                                            \n");
    fprintf(pager,"General topics:                                                             \n");
    fprintf(pager,"  general  - general neo info                                               \n");
    fprintf(pager,"  version  - version specific information                                   \n");
    fprintf(pager,"  syntax   - the location syntax                                            \n");
    fprintf(pager,"  args     - arguments to neo                                               \n");
    fprintf(pager,"  sample   - sample commands                                                \n");
    fprintf(pager,"                                                                            \n");
    fprintf(pager,"Neo commands:                                                               \n");
    fprintf(pager,"  arpfind   - translate ip to macaddress                                    \n");
    fprintf(pager,"  locate    - locate a host                                                 \n");
    fprintf(pager,"  port      - get the status of a port, turn it on or off, find the         \n");
    fprintf(pager,"                hosts on a port etc.                                        \n");
    fprintf(pager,"  device    - device specific commands                                      \n");
    fprintf(pager,"  stats     - network usage statistics                                      \n");
    fprintf(pager,"  location  - print attributes of a location                                \n");
    fprintf(pager,"  set       - set variables                                                 \n");
    fprintf(pager,"  print     - print variables                                               \n");
    fprintf(pager,"  hostinfo  - get host information                                          \n");
    fprintf(pager,"  exec      - exec a shell command                                          \n");
    fprintf(pager,"  help      - help                                                          \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * ARPFIND
   */
  if (!strcmp(argv[0], "arpfind")) {
    fprintf(pager,"NEO HELP                                                           (arpfind)\n");
    fprintf(pager,"                                                                            \n");
    fprintf(pager,"NAME                                                                        \n");
    fprintf(pager,"   arpfind - look up the mac address of a host in the router's arpcache     \n");
    fprintf(pager,"                                                                            \n");
    fprintf(pager,"SYNOPSYS                                                                    \n");
#ifdef MIT    
    fprintf(pager,"   arpfind <hostname> [<router>]                                            \n");
#else
    fprintf(pager,"   arpfind <hostname> <router>                                              \n");
#endif
    fprintf(pager,"                                                                            \n");
    fprintf(pager,"DESCRIPTION                                                                 \n");
#ifdef MIT
    fprintf(pager,"   Arpfind will look up the mac address of <hostname> on the appropriate    \n");
    fprintf(pager,"   router or on the router specified.  Note that because this build has     \n");
    fprintf(pager,"   MIT extensions the <router> argument is not required, as it is at other  \n");
    fprintf(pager,"   places.                                                                  \n");
#else
    fprintf(pager,"   Arpfind will look up the mac address of <hostname> on <router>           \n");
#endif
    fprintf(pager,"                                                                            \n");
    fprintf(pager,"OPTIONS                                                                     \n");
    fprintf(pager,"                                                                            \n");
    fprintf(pager,"BUGS                                                                        \n");
    fprintf(pager,"   Doesn't do reverse lookups yet                                           \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * LOCATE
   */
  if (!strcmp(argv[0], "locate")) {
    fprintf(pager,"NEO HELP                                                           (locate)\n");
    fprintf(pager,"                                                                           \n");
    fprintf(pager,"NAME                                                                       \n");
    fprintf(pager,"   locate - locate a node on a set of devices                              \n");
    fprintf(pager,"                                                                           \n");
    fprintf(pager,"SYNOPSYS                                                                   \n");
    fprintf(pager,"   locate [-v] <mac> <location>                                            \n");
    fprintf(pager,"                                                                           \n");
    fprintf(pager,"DESCRIPTION                                                                \n");
    fprintf(pager,"   locate will search the devices in <location> for the presence           \n");
    fprintf(pager,"   of the given mac address on any of its ports.                           \n");
    fprintf(pager,"                                                                           \n");
    fprintf(pager,"OPTIONS                                                                    \n");
    fprintf(pager,"   -v  Verbose.  Display the name of each device as it's being searched    \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * PORT
   */
  if (!strcmp(argv[0], "port")) {
    fprintf(pager,"NEO HELP                                                           (port)\n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"NAME                                                                     \n");
    fprintf(pager,"   port - check the status of a port, enable it, disable, or lookup hosts\n");
    fprintf(pager,"          using it                                                       \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"SYNOPSYS                                                                 \n");
    fprintf(pager,"   port status|enable|disable <location>                                 \n");
    fprintf(pager,"   port search <location>                                                \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"DESCRIPTION                                                              \n");
    fprintf(pager,"   The port command can modify or check the admin status of a drop with  \n");
    fprintf(pager,"   the 'enable', 'disable' and 'status' sub-commands.                    \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"   The 'search' sub-command will return the mac addresses a device as    \n");
    fprintf(pager,"   seen on a port.                                                       \n");
    fprintf(pager,"   a port.                                                               \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"OPTIONS                                                                  \n");
    fprintf(pager,"   enable   turn the drop on                                             \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"   disable  turn the drop off                                            \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"   status   check the status of the drop                                 \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"   search   search for the mac addresses on this port                    \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"BUGS                                                                     \n");
    fprintf(pager,"   Search is not yet supported, but is implemented for some devices.     \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"   There are other port commands to be done, i.e. duplex, bulk-disable,  \n");
    fprintf(pager,"   bulk-enable, info, etc.                                               \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * STATS
   */
  if (!strcmp(argv[0], "stats")) {
    fprintf(pager,"NEO HELP                                                           (stats)\n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"NAME                                                                      \n");
    fprintf(pager,"   stats - get network statistics.                                        \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"SYNOPSYS                                                                  \n");
    fprintf(pager,"   stats <location>                                                       \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"DESCRIPTION                                                               \n");
    fprintf(pager,"   The stats command gets network statics on a device or a group of       \n");
    fprintf(pager,"   devices.  This may include bandwidth usage, pps stats, link status etc.\n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"OPTIONS                                                                   \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"BUGS                                                                      \n");
    fprintf(pager,"   Stats does not yet work across multiple devices at the same time.      \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * LOCATION
   */
  if (!strcmp(argv[0], "location")) {
    fprintf(pager,"NEO HELP                                                        (location)\n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"NAME                                                                      \n");
    fprintf(pager,"   location - print attributes of a location                              \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"SYNOPSYS                                                                  \n");
    fprintf(pager,"   location print <location>                                              \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"DESCRIPTION                                                               \n");
    fprintf(pager,"   The location print command will display all devices associated with    \n");
    fprintf(pager,"   the named location.  For example 'location print @k:10.10' will list   \n");
    fprintf(pager,"   all the devices in the keys file for 10.10.  For another example:      \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"      neo: location print 2/4@foo,bar                                     \n");
    fprintf(pager,"      Devices (2) are:                                                    \n");
    fprintf(pager,"      foo                                                                 \n");
    fprintf(pager,"      bar                                                                 \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"OPTIONS                                                                   \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * PRINT
   */
  if (!strcmp(argv[0], "print")) {
    fprintf(pager,"NEO HELP                                                           (print)\n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"NAME                                                                      \n");
    fprintf(pager,"   print - print a variable's value.                                      \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"SYNOPSYS                                                                  \n");
    fprintf(pager,"   print [ <variable> ]                                                   \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"DESCRIPTION                                                               \n");
    fprintf(pager,"   Print the value of <variable>.  If no variable is specified then all   \n");
    fprintf(pager,"   variable values are printed.                                           \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"   Legal variables are:                                                   \n");
    fprintf(pager,"      writecom                                                            \n");
    fprintf(pager,"      readcom                                                             \n");
    fprintf(pager,"      keyfile                                                             \n");
    fprintf(pager,"      timeout                                                             \n");
    fprintf(pager,"      statsdelay                                                          \n");
    fprintf(pager,"      macmode                                                             \n");
    fprintf(pager,"      verbose                                                             \n");
    fprintf(pager,"      debug                                                               \n");
    fprintf(pager,"      version                                                             \n");
    fprintf(pager,"                                                                          \n");
    fprintf(pager,"   For more information on the meaning of each variable consult           \n");
    fprintf(pager,"   'help set'.                                                            \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * SET
   */
  if (!strcmp(argv[0], "set")) {
    fprintf(pager,"NEO HELP                                                           (set)\n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"NAME                                                                    \n");
    fprintf(pager,"   set - set a variable's value.                                        \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"SYNOPSYS                                                                \n");
    fprintf(pager,"   set <variable> [ <value> ]                                           \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"DESCRIPTION                                                             \n");
    fprintf(pager,"   Set the value of a <variable> to <value>.  If the variable is either \n");
    fprintf(pager,"   writecom or readcom you may leave the value off the line and you'll  \n");
    fprintf(pager,"   be prompted to enter it on a line where your keystrokes will not be  \n");
    fprintf(pager,"   echoed.                                                              \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"   Legal variables are:                                                 \n");
    fprintf(pager,"      writecom                                                          \n");
    fprintf(pager,"      readcom                                                           \n");
    fprintf(pager,"      keyfile                                                           \n");
    fprintf(pager,"      timeout                                                           \n");
    fprintf(pager,"      statsdelay                                                        \n");
    fprintf(pager,"      macmode                                                           \n");
    fprintf(pager,"      verbose                                                           \n");
    fprintf(pager,"      debug                                                             \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"   writecom:                                                            \n");
    fprintf(pager,"      This sets the SNMP community used for write requests.  It can be  \n");
    fprintf(pager,"      set to any string and is case sensitive.                          \n");
    fprintf(pager,"   readcom:                                                             \n");
    fprintf(pager,"      This sets the SNMP community used for read requests.  It can be   \n");
    fprintf(pager,"      set to any string and is case sensitive.                          \n");
    fprintf(pager,"   keyfile:                                                             \n");
    fprintf(pager,"      This sets the name of the file used to do keyfile lookups         \n");
    fprintf(pager,"      with @k:<key>.  See 'help syntax' for further information on the  \n");
    fprintf(pager,"      keyfile.                                                          \n");
    fprintf(pager,"   timeout:                                                             \n");
    fprintf(pager,"      This sets the time in seconds that neo should wait for responses  \n");
    fprintf(pager,"      to SNMP queries.  The default is 8 seconds.                       \n");
    fprintf(pager,"   statsdelay:                                                          \n");
    fprintf(pager,"      This sets the minimum amount of time in seconds for the delta in  \n");
    fprintf(pager,"      stats gathering.  The default is 2 seconds.                       \n");
    fprintf(pager,"   macmode:                                                             \n");
    fprintf(pager,"      This sets the format that should be used for displaying mac       \n");
    fprintf(pager,"      addresses.  Valid options are 'standard', 'cisco', and 'dash'.    \n");
    fprintf(pager,"   verbose:                                                             \n");
    fprintf(pager,"      This instructs Neo to present more verbose output while execting  \n");
    fprintf(pager,"      commands.                                                         \n");
    fprintf(pager,"   debug:                                                               \n");
    fprintf(pager,"      This enables even more detailed output while running commands, in \n");
    fprintf(pager,"      to help find bugs in Neo.                                         \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * SAMPLE
   */
  if (!strcmp(argv[0], "sample")) {
    fprintf(pager,"NEO HELP                                                        (sample)\n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"Locating a host:                                                        \n");
    fprintf(pager,"                                                                        \n");
#ifdef MIT
    fprintf(pager,"  neo:  arpfind myhost                                                  \n");
#else
    fprintf(pager,"  neo:  arpfind myhost myrouter                                         \n");
#endif
    fprintf(pager,"  10.10.0.1 says 10.10.1.101 is 08:00:20:7D:48:1B                       \n");
    fprintf(pager,"  neo:  locate 08:00:20:7D:48:1B @k:10.10                               \n");
    fprintf(pager,"  Found on 2/14@SWITCH-ENTRY.EXAMPLE.COM                                \n");
    fprintf(pager,"  Found on 3@SWITCH-5.EXAMPLE.COM                                       \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"Disabling a drop:                                                       \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"  neo:  set writecom foobar                                             \n");
    fprintf(pager,"  neo:  port disable 3@switch-5.example.com                             \n");
    fprintf(pager,"  3@SWITCH-5.EXAMPLE.COM disabled                                       \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"Getting a sumary of connections on a device:                            \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"  neo:  device summary myrepeater                                       \n");
    fprintf(pager,"  Board summary:                                                        \n");
    fprintf(pager,"   b type       sg  ul ultype                                           \n");
    fprintf(pager,"  -------------------------------                                       \n");
    fprintf(pager,"   1 Mgmt        1                                                      \n");
    fprintf(pager,"   2 RJ21-Top*   1  10 UTP-MAU                                          \n");
    fprintf(pager,"   3 RJ21-Bot*   1                                                      \n");
    fprintf(pager,"   4 RJ21-Top*   2                                                      \n");
    fprintf(pager,"   5 RJ21-Bot*   2                                                      \n");
    fprintf(pager,"   6 RJ21-Top*   2  10 UTP-MAU                                          \n");
    fprintf(pager,"   7 RJ21-Bot*   2                                                      \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"  neo:  dev sum 2/*@myrepeater                                          \n");
    fprintf(pager,"  Port summary:                                                         \n");
    fprintf(pager,"   p type  u lnk adm ap                                                 \n");
    fprintf(pager,"  ---------------------                                                 \n");
    fprintf(pager,"   1 10T       -  On                                                    \n");
    fprintf(pager,"   2 10T      10  On                                                    \n");
    fprintf(pager,"   3 10T       -  On                                                    \n");
    fprintf(pager,"   4 10T      10  On                                                    \n");
    fprintf(pager,"   5 10T      10  On                                                    \n");
    fprintf(pager,"   6 10T      10  On                                                    \n");
    fprintf(pager,"   7 10T      10  On                                                    \n");
    fprintf(pager,"   8 10T      10  On                                                    \n");
    fprintf(pager,"   9 10T      10  On                                                    \n");
    fprintf(pager,"  10 10T       -  On                                                    \n");
    fprintf(pager,"  11 10T      10  On                                                    \n");
    fprintf(pager,"  12 10T      10  On                                                    \n");
    fprintf(pager,"  13 UTP-M *  10  On                                                    \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * SYNTAX
   */
  if (!strcmp(argv[0], "syntax")) {
    fprintf(pager,"NEO HELP                                                        (syntax)\n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"The location syntax is made up of two parts: the board and/or port and  \n");
    fprintf(pager,"host or hosts involved.  For example board 5, port 2 on                 \n");
    fprintf(pager,"switch-entry.example.com is represented as:                             \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"         5/2@switch-entry.example.com                                   \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"If the device doesn't have any boards, then only the port is given, as  \n");
    fprintf(pager,"in:                                                                     \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"          3@switch-5.example.com                                        \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"If you need to refer to a board itself and not any particular port on   \n");
    fprintf(pager,"that board use:                                                         \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"         5/@switch-entry.example.com                                    \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"Which would refer to board 5 on that device.  You can also use an       \n");
    fprintf(pager,"asterisk ('*') to denote multiple boards or ports.  To refer to all     \n");
    fprintf(pager,"ports on a device with no boards you would use:                         \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"          *@switch-5.example.com                                        \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"If the device does have boards then you can do a number of different    \n");
    fprintf(pager,"things to refer to different groups of ports or boards.  I.e.:          \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"          */@switch-entry            all boards                         \n");
    fprintf(pager,"         */*@switch-entry            all ports (on all boards)          \n");
    fprintf(pager,"         5/*@switch-entry            all ports on board 5               \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"You can also use a period ('.') in place of an asterisk.                \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"If you need to specify a host, and not any boards or ports on that host,\n");
    fprintf(pager,"just use:                                                               \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"         @switch-5.example.com                                          \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"If you need to refer to a small list of hosts you can use:              \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"         @switch-5,switch-6,switch-7                                    \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"If you need to refer to a list of hosts from a file:                    \n");
    fprintf(pager,"         @f:/var/tmp/hostlist                                           \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"And if you need to refer to hosts matching a key from a keyfile:        \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"         @k:10.10                                                       \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"You can also mix and match the above board and port possiblities with   \n");
    fprintf(pager,"the host options.  For example:                                         \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"         3/*@k:10.10                                                    \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"would be all ports on board 3 on every device on 10.10.  Which is kind  \n");
    fprintf(pager,"of silly, granted.                                                      \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * GENERAL
   */
  if (!strcmp(argv[0], "general")) {
    fprintf(pager,"NEO HELP                                                       (general)\n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"Neo is a free network administration tool.  It works across multiple    \n");
    fprintf(pager,"types of network hardware and is designed so that it is easy to add     \n");
    fprintf(pager,"support for new types of hardware.  If you write modules for new        \n");
    fprintf(pager,"hardware or have hardware I can test against for new modules send mail  \n");
    fprintf(pager,"to bug-neo@mit.edu                                                      \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"This is Neo version %s  The Neo distribution is                         \n", neo_global_get_version(g));
    fprintf(pager,"available at http://web.mit.edu/ktools/  Check there for new            \n");
    fprintf(pager,"releases and other information.                                         \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"Neo was written by James Kretchmar at the Massachusetts Institute of    \n");
    fprintf(pager,"Technology.                                                             \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"Copyright 2003 by the Massachusetts Institute of Technology.            \n");
    fprintf(pager,"                                                                        \n");
    fprintf(pager,"Permission to use, copy, modify, and distribute this                    \n");
    fprintf(pager,"software and its documentation for any purpose and without              \n");
    fprintf(pager,"fee is hereby granted, provided that the above copyright                \n");
    fprintf(pager,"notice appear in all copies and that both that copyright                \n");
    fprintf(pager,"notice and this permission notice appear in supporting                  \n");
    fprintf(pager,"documentation, and that the name of M.I.T. not be used in               \n");
    fprintf(pager,"advertising or publicity pertaining to distribution of the              \n");
    fprintf(pager,"software without specific, written prior permission.                    \n");
    fprintf(pager,"M.I.T. makes no representations about the suitability of                \n");
    fprintf(pager,"this software for any purpose.  It is provided \"as is\"                  \n");
    fprintf(pager,"without express or implied warranty.                                    \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * DEVICE
   */
  if (!strcmp(argv[0], "device")) {
    fprintf(pager,"\n");
    fprintf(pager,"NEO HELP                                                         (device)\n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"NAME                                                                     \n");
    fprintf(pager,"   device - get information about a device                               \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"SYNOPSYS                                                                 \n");
    fprintf(pager,"   device type <location>                                                \n");
    fprintf(pager,"   device summary <location>                                             \n");
    fprintf(pager,"   device capabilities <location>                                        \n");
    fprintf(pager,"   device uplink <location>                                              \n");
    fprintf(pager,"   device set <location> system contact <contact>                        \n");
    fprintf(pager,"   device set <location> system name <name>                              \n");
    fprintf(pager,"   device set <location> system location <location>                      \n");
    fprintf(pager,"   device reset <location>                                               \n");
    fprintf(pager,"   device download image|config <location> <tftphost> <filename>         \n");
    fprintf(pager,"   device upload image|config <location> <tftphost> <filename>           \n");
    fprintf(pager,"   device info <location>                                                \n");
    fprintf(pager,"   device info weather <location>                                        \n");
    fprintf(pager,"   device info power <location>                                          \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"DESCRIPTION                                                              \n");
    fprintf(pager,"   The 'device type' command will display what kind of device the        \n");
    fprintf(pager,"   location is.                                                          \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"   The 'device summary' command will print a summary of the connections  \n");
    fprintf(pager,"   in use at the location.                                               \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * ARGS
   */
  if (!strcmp(argv[0], "args")) {
    fprintf(pager,"NEO HELP                                                           (args)\n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"  Neo can execute any of it's commands on the command line.  In addition \n");
    fprintf(pager,"  the following command line arguments are avaliable:                    \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"  -k <keyfile>                                                           \n");
    fprintf(pager,"        Set the kefyile to <keyfile>.                                    \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"  -w <community>                                                         \n");
    fprintf(pager,"        Set the write community to <community>.                          \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"  -r <community>                                                         \n");
    fprintf(pager,"        Set the read community to <community>.                           \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"  -c <community>                                                         \n");
    fprintf(pager,"        Set both the read and write communities to <community>           \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * HOSTINFO
   */
  if (!strcmp(argv[0], "hostinfo")) {
    fprintf(pager,"NEO HELP                                                       (hostinfo)\n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"NAME                                                                     \n");
    fprintf(pager,"   hostinfo - look information on a host or ip address.                  \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"SYNOPSYS                                                                 \n");
    fprintf(pager,"   hostinfo <host>                                                       \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"DESCRIPTION                                                              \n");
    fprintf(pager,"   Hostinfo will return the ip address, hostname and hostinfo of the     \n");
    fprintf(pager,"   specified host.                                                       \n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * EXEC
   */
  if (!strcmp(argv[0], "exec")) {
    fprintf(pager,"NEO HELP                                                           (exec)\n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"NAME                                                                     \n");
    fprintf(pager,"   exec - exec a shell command.                                          \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"SYNOPSYS\n");
    fprintf(pager,"   exec <command>\n");
    fprintf(pager,"\n");
    fprintf(pager,"DESCRIPTION\n");
    fprintf(pager,"   exec runs the specified command in a subshell.\n");
    fprintf(pager,"\n");
    if (pstr) pclose(pager);
    return;
  }

  /*
   * VERSION
   */
  if (!strcmp(argv[0], "version")) {
    fprintf(pager,"NEO HELP                                                        (version)\n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"This is Neo version %s.  Major new features for\n", neo_global_get_version(g));
    fprintf(pager,"recent versions are:                                                     \n");
    fprintf(pager,"                                                                         \n");
    fprintf(pager,"     xx-xx-xx-xx-xx-xx is now an acceptable mac format                   \n");
    fprintf(pager,"     'dev capabilities' (or 'dev cap') command.                          \n");
    fprintf(pager,"     'dev uplink' command.                                               \n");
    fprintf(pager,"     Auto uplink detection.                                              \n");
    if (pstr) pclose(pager);
    return;
  }

  fprintf(pager, "No help is avaliable on that topic.\n");
  if (pstr) pclose(pager);
  return;
}
