#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "neo.h"

int neo_function_standardbridge_get_ifnum_from_port(neo_device *d, int port)
{
  char obj[1024], val[1024];
  int ret;
  
  sprintf(obj, ".1.3.6.1.2.1.17.1.4.1.2.%i", port);
  ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, val);
  if (ret) {
    return(-1);
  }
  return(atoi(val));
}

/* expensive on SNMP requests */
int neo_function_standardbridge_get_port_from_ifnum(neo_device *d, int ifnum)
{
  neo_snmpresponselist *rl;
  int i, j;
  char *ptr, *val;
  
  rl=neo_snmp_many_requests_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.17.1.4.1.2");
  if (rl==NULL) {
    if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
      printf("Could not contact device %s\n", neo_device_get_name(d));
      neo_snmpresponselist_free_all(rl);
      return(-1);
    } else {
      printf("Error talking to device %s\n", neo_device_get_name(d));
      neo_snmpresponselist_free_all(rl);
      return(-1);
    }
  }

  j=neo_snmpresponselist_get_size(rl);
  for (i=0; i<j; i++) {
    val=neo_snmpresponselist_get_val(rl, i);
    if (atoi(val)==ifnum) {
      ptr=neo_snmpresponselist_get_var(rl, i)+24;
      neo_snmpresponselist_free_all(rl);
      return(atoi(ptr));
    }
  }
  neo_snmpresponselist_free_all(rl);
  return(-1);
}

/* Return the IfIndex the given searchable is located on, or -1
 * if there is an error.  Does not touch neo_error. */
int neo_function_standardbridge_get_port_from_mac(neo_device *d, neo_searchable *s)
{
  neo_macaddr *m;
  char obj[1024], tmp[300], val[30];
  int ret;

  m=neo_searchable_get_macaddr(s);
  neo_macaddr_to_dotted_ints_string(m, tmp);

  /* mib-2.dot1dBridge.dot1dTp.dot1dTpFdbTable.dot1dTpFdbEntry.dot1dTpFdbPort */
  strcpy(obj, ".1.3.6.1.2.1.17.4.3.1.2.");
  strcat(obj, tmp);

  ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, val);
  if (ret) {
    return(ret);
  }

  ret=neo_function_standardbridge_get_ifnum_from_port(d, atoi(val));
  if (ret<0) return(-1);
  return(ret);
}


int neo_function_standardbridge_portsearch(neo_device *d, int ifnum)
{
  neo_macaddr m;
  neo_snmpresponselist *rl;
  int i, j, ret, x, port;
  char *var, *val, buff[50], val2[1024], obj[1024];

  /* first figure out the standard bridge port number from the ifnum */
  port=neo_function_standardbridge_get_port_from_ifnum(d, ifnum);
  
  /* mib-2.dot1dBridge.dot1dTp.dot1dTpFdbTable.dot1dTpFdbEntry.dot1dTpFdbPort */
  rl=neo_snmp_many_requests_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.17.4.3.1.2");
  if (rl==NULL) {
    if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
      printf("Could not contact device %s\n", neo_device_get_name(d));
      neo_snmpresponselist_free_all(rl);
      return(-1);
    } else {
      printf("Error talking to device %s\n", neo_device_get_name(d));
      return(0);
    }
  }

  j=neo_snmpresponselist_get_size(rl);
  for (i=0; i<j; i++) {
    val=neo_snmpresponselist_get_val(rl, i);
    if (atoi(val)==port) {
      /* check that it's not self known */
      var=neo_snmpresponselist_get_var(rl, i);
      strcpy(obj, var);
      obj[22]='3';
      ret=neo_snmp_request_by_variable(neo_device_get_name(d), obj, val2);
      x=atoi(val2);
      if (x==4) {
	/* it's the interface address, don't display it */
	continue;
      }
      neo_macaddr_create_from_dotted_ints(&m, var+strlen(".1.3.6.1.2.1.17.4.3.1.2."));
      neo_macaddr_to_string(&m, buff);
      printf("%s", buff);
      if (x==3 || x==0) {
	printf("\n");
      } else {
	printf("     (status %i)\n", atoi(val2));
      }

    }
  }
  return(0);
}

int neo_function_standardbridge_portsed(neo_location *loc, int command, int ifnum)
{
  char obj[1024], val[20];
  int a, ret;
  char *name;

  name=neo_device_get_name(neo_location_get_first_device(loc));

  /* mib-2.dot1dBridge.dot1dStp.dot1dStpPortTable.dot1dStpPortEntry.dot1dStpPortEnable */
  sprintf(obj, ".1.3.6.1.2.1.17.2.15.1.4.%i", ifnum);

  if (command == NEO_COMMAND_PORT_STATUS) {
    ret=neo_snmp_request_by_variable(name, obj, val);
    if (ret) {
      if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
	printf("Could not contact device %s\n.", name);
      } else {
	printf("Error getting port status for %s.\n", neo_location_get_instr(loc));
      }
      return(NEO_COMMAND_PORT_IS_UNKNOWN);
    }
    a=atoi(val);
    if (a==1) return(NEO_COMMAND_PORT_IS_ENABLED);
    if (a==2) return(NEO_COMMAND_PORT_IS_DISABLED);
    return(NEO_COMMAND_PORT_IS_UNKNOWN);
  } else if (command == NEO_COMMAND_PORT_ENABLE) {
    ret=neo_snmp_set_by_variable(name, obj, "int", "1");
    if (ret) {
      if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
	printf("Could not contact device %s\n", name);
      } else {
	printf("Error getting port status for %s.\n",neo_location_get_instr(loc));
      }
      return(NEO_COMMAND_PORT_IS_UNKNOWN);
    }
    return(NEO_COMMAND_PORT_IS_ENABLED);
  } else if (command == NEO_COMMAND_PORT_DISABLE) {
    ret=neo_snmp_set_by_variable(name, obj, "int", "2");
    if (ret) {
      if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
	printf("Could not contact device %s\n", name);
      } else {
	printf("Error getting port status for %s.\n", neo_location_get_instr(loc));
      }
      return(NEO_COMMAND_PORT_IS_UNKNOWN);
    }
    return(NEO_COMMAND_PORT_IS_DISABLED);
  }
  return(0);
}

int neo_function_standardbridge_stats(neo_device *d, neo_statstransfer *st, int ifnum)
{
  char obj[1024], val[1024];
  int ret;
  unsigned long x;
  char *name;

  name=neo_device_get_name(d);
  
  /* interfaces.ifTable.ifEntry.ifInOctets.%i */
  sprintf(obj, ".1.3.6.1.2.1.2.2.1.10.%i", ifnum);
  ret=neo_snmp_request_by_variable(name, obj, val);
  if (ret && (neo_error!=NEO_ERR_SNMP_NOVAR)) {
    neo_statstransfer_set_generic(st, NEO_ST_INKBS, NEO_ST_ERROR, 0);
  } else {
    x=(atol(val)*8)/1000;
    neo_statstransfer_set_generic(st, NEO_ST_INKBS, NEO_ST_INT, x);
  }
  neo_statstransfer_set_clk_inkbs(st);

  /* interfaces.ifTable.ifEntry.ifOutOctets.%i */
  sprintf(obj, ".1.3.6.1.2.1.2.2.1.16.%i", ifnum);
  ret=neo_snmp_request_by_variable(name, obj, val);
  if (ret && (neo_error!=NEO_ERR_SNMP_NOVAR)) {
    neo_statstransfer_set_generic(st, NEO_ST_OUTKBS, NEO_ST_ERROR, 0);
  } else {
    x=(atol(val)*8)/1000;
    neo_statstransfer_set_generic(st, NEO_ST_OUTKBS, NEO_ST_INT, x);
  }
  neo_statstransfer_set_clk_outkbs(st);

  /* interfaces.ifTable.ifEntry.ifInUcastPkts.%i */
  sprintf(obj, ".1.3.6.1.2.1.2.2.1.11.%i", ifnum);
  ret=neo_snmp_request_by_variable(name, obj, val);
  if (ret && (neo_error!=NEO_ERR_SNMP_NOVAR)) {
    neo_statstransfer_set_generic(st, NEO_ST_INPPSU, NEO_ST_ERROR, 0);
  } else {
    neo_statstransfer_set_generic(st, NEO_ST_INPPSU, NEO_ST_INT, atol(val));
  }
  neo_statstransfer_set_clk_inppsu(st);

  /* interfaces.ifTable.ifEntry.ifOutUcastPkts.%i */
  sprintf(obj, ".1.3.6.1.2.1.2.2.1.17.%i", ifnum);
  ret=neo_snmp_request_by_variable(name, obj, val);
  if (ret && (neo_error!=NEO_ERR_SNMP_NOVAR)) {
    neo_statstransfer_set_generic(st, NEO_ST_OUTPPSU, NEO_ST_ERROR, 0);
  } else {
    neo_statstransfer_set_generic(st, NEO_ST_OUTPPSU, NEO_ST_INT, atol(val));
  }
  neo_statstransfer_set_clk_outppsu(st);
  
  /* interfaces.ifTable.ifEntry.ifInNUcastPkts.%i */
  sprintf(obj, ".1.3.6.1.2.1.2.2.1.12.%i", ifnum);
  ret=neo_snmp_request_by_variable(name, obj, val);
  if (ret && (neo_error!=NEO_ERR_SNMP_NOVAR)) {
    neo_statstransfer_set_generic(st, NEO_ST_INPPSNU, NEO_ST_ERROR, 0);
  } else {
    neo_statstransfer_set_generic(st, NEO_ST_INPPSNU, NEO_ST_INT, atol(val));
  }
  neo_statstransfer_set_clk_inppsnu(st);

  /* interfaces.ifTable.ifEntry.ifOutNUcastPkts.%i */
  sprintf(obj, ".1.3.6.1.2.1.2.2.1.18.%i", ifnum);
  ret=neo_snmp_request_by_variable(name, obj, val);
  if (ret && (neo_error!=NEO_ERR_SNMP_NOVAR)) {
    neo_statstransfer_set_generic(st, NEO_ST_OUTPPSNU, NEO_ST_ERROR, 0);
  } else {
    x=atol(val);
    neo_statstransfer_set_generic(st, NEO_ST_OUTPPSNU, NEO_ST_INT, atol(val));
  }
  neo_statstransfer_set_clk_outppsnu(st);

  /* interfaces.ifTable.ifEntry.ifInErrors.%i */
  sprintf(obj, ".1.3.6.1.2.1.2.2.1.14.%i", ifnum);
  ret=neo_snmp_request_by_variable(name, obj, val);
  if (ret && (neo_error!=NEO_ERR_SNMP_NOVAR)) {
    neo_statstransfer_set_generic(st, NEO_ST_INERRPS, NEO_ST_ERROR, 0);
  } else {
    neo_statstransfer_set_generic(st, NEO_ST_INERRPS, NEO_ST_INT, atol(val));
  }
  neo_statstransfer_set_clk_inerrps(st);

  /* interfaces.ifTable.ifEntry.ifOutErrors.%i */
  sprintf(obj, ".1.3.6.1.2.1.2.2.1.20.%i", ifnum);
  ret=neo_snmp_request_by_variable(name, obj, val);
  if (ret && (neo_error!=NEO_ERR_SNMP_NOVAR)) {
    neo_statstransfer_set_generic(st, NEO_ST_OUTERRPS, NEO_ST_ERROR, 0);
  } else {
    neo_statstransfer_set_generic(st, NEO_ST_OUTERRPS, NEO_ST_INT, atol(val));
  }
  neo_statstransfer_set_clk_outerrps(st);

  return(0);
}

/* Set the bridgeindex for every port in the layout.  If the device
 * does not support the standard bridge mib return -1, otherwise
 * return 0.  If the layout already contains ifindex values, they will
 * be used, if not, they will be calculated but not filled in.
 */
int neo_function_standardbridge_set_layout_bridgeports(neo_device *d, neo_layout *lo)
{
  neo_snmpresponselist *rl;
  char *ptr, *val;
  int i, j, ifindex, x, y;
  neo_boardport bp;

  /* get the mapping of bridgeindex -> ifindex into rl */
  if (neo_global_is_debug(g)) {
    printf("function_standardbridge_set_layout_bridgeports: starting dump for %s.\n",
	   neo_device_get_name(d));
  }
  rl=neo_snmp_many_requests_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.17.1.4.1.2");
  if (rl==NULL) {
    if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
      printf("Could not contact device %s\n", neo_device_get_name(d));
      neo_snmpresponselist_free_all(rl);
      return(-1);
    } else {
      if (neo_global_is_debug(g)) printf("No bridgemib on device %s\n", neo_device_get_name(d));
      neo_snmpresponselist_free_all(rl);
      return(-1);
    }
  }

  /* go through every port on the layout */
  if (!neo_layout_has_boards(lo)) {
    /* no boards */

    if (neo_global_is_debug(g)) {
      printf("function_standardbridge_set_layout_bridgeports: %s does not have boards.\n", neo_device_get_name(d));
    }
      
    j=neo_layout_get_numports(lo);
    for (i=0; i<j; i++) {
      /* get the ifindex for this port */
      ifindex=neo_layout_get_port_ifindex(lo, i+1);
      if (ifindex==-1) {
	neo_boardport_create_from_port_int(&bp, i+1);
	ifindex=neo_device_get_ifindex_from_boardport(d, &bp);
	neo_boardport_free(&bp);
      }

      /* lookup ifindex in rl */
      y=neo_snmpresponselist_get_size(rl);
      for (x=0; x<y; x++) {
	val=neo_snmpresponselist_get_val(rl, x);
	if (atoi(val)==ifindex) {
	  ptr=neo_snmpresponselist_get_var(rl, x)+24;
	  /* atoi(ptr) is the bridgeindex corresponding to that if ifindex
	   *  and therefore that port */
	  neo_layout_set_port_bridgeindex(lo, i+1, atoi(ptr));
	  if (neo_global_is_debug(g)) {
	    printf("function_standardbridge_set_layout_bridgeports: setting bridgeindex %i for port %i (ifindex %i) on %s.\n",
		   atoi(ptr),
		   i+1,
		   ifindex,
		   neo_device_get_name(d));
	  }
	}
      }
      /* we didn't find it, just do nothing */
    }
  } else {
    /* boards */

    if (neo_global_is_debug(g)) {
      printf("function_standardbridge_set_layout_bridgeports: %s has boards.\n", neo_device_get_name(d));
    }

    /* NEED TO IMPLEMENT THIS */
    /* RESUME CODING HERE */

  }
  neo_snmpresponselist_free_all(rl);
  return(0);
}
