#include "neo.h"
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>

  enum {
    HUB_ASN_2072,
    HUB_ASN_1012,
    HUB_ASN_1012_BDG,
    HUB_ASN_2024,
    HUB_ASN_5324,
    HUB_ASN_6216,
    
    HUB_CTN_MRXI,
    HUB_CTN_2200,

    HUB_EXT_SUM1I,
    HUB_EXT_SUM24,
    HUB_EXT_SUM48,
    HUB_EXT_ALP,
    HUB_EXT_BD,

    HUB_HP_2524,

    HUB_CIS_C2948,
    HUB_CIS_C2950,
    HUB_CIS_C4000,
    HUB_CIS_C5000,
    HUB_CIS_C5002,
    HUB_CIS_C5500,

    RTR_CIS_2500_IOS,
    RTR_CIS_2600_IOS,
    RTR_CIS_3500_IOS,
    RTR_CIS_4500_IOS,
    RTR_CIS_7500_IOS,
    RTR_CIS_6509_IOS,
    RTR_CIS_8500_IOS,
    
    HUB_BST_450F,

    HUB_LUC_AP,
    HUB_LUC_AP1000,
    HUB_LUC_AP2000,
    HUB_CIS_AP,
    HUB_AVA_AP3,

    UPS_APC,
    UPS_MGE_COMET,


    PRINTER_HP_GENERIC,

    GENERIC_DEVICE
  };

int neo_device_create_by_name(neo_device *d, char *name)
{
  d->name=strdup(name);
  d->devname=NULL;
  d->boardlist=NULL;
  d->portlist=NULL;
  d->constset=0;
  d->layoutset=0;
  neo_layout_create(&(d->layout));
  return(0);
}

int neo_device_set_const_from_probe(neo_device *d, neo_global *g)
{
  int ret;
  char descr[1024];

  /* system.sysObjectID.0 */
  if (!d->name) return(-1);
  /* system.sysObjectID.0 */
  ret=neo_snmp_request_by_variable(d->name, ".1.3.6.1.2.1.1.2.0", descr);
  if (ret) {
    if (neo_error==NEO_ERR_SNMP_NOHOST) {
      printf("No such host %s.\n", neo_device_get_name(d));
      return(-1);
    } else {
      if (neo_global_is_verbose(g) || neo_global_is_debug(g)) {
	printf("Error setting device constants for %s\n", d->name);
      }
      return(-1);
    }
  }
  ret=_neo_device_setconsts_from_oid(d, descr);
  if (ret) {
    return(-1);
  }

  return(0);
}

neo_layout *neo_device_get_layout(neo_device *d)
{
  return(&(d->layout));
}

int neo_device_is_layout_set(neo_device *d)
{
  if (d->layoutset) return(1);
  return(0);
}

int neo_device_set_layout(neo_device *d)
{
  int ret;

  if (!d->func_set_layout) {
    neo_error=NEO_ERR_CONTACT;
    return(-1);
  }
  ret=(* d->func_set_layout)(d);
  d->layoutset=1;
  return(ret);
}

void neo_device_set_layout_set(neo_device *d)
{
  d->layoutset=1;
}

int neo_device_create_and_set_const(neo_device *d, neo_global *g, char *name)
{
  neo_device_create_by_name(d, name);
  neo_device_set_const_from_probe(d, g);
  return(0);
}

int neo_device_has_boards(neo_device *d)
{
  return(d->hasboards);
}

char *neo_device_get_name(neo_device *d)
{
  return(d->name);
}

char *neo_device_get_devname(neo_device *d)
{
  if (d->devname) return(d->devname);
  return("Unset");
}

char *neo_device_get_fullname(neo_device *d)
{
  if (d->fullname) return(d->fullname);
  return("Unset");
}

neo_boardlist *neo_device_get_boardlist(neo_device *d)
{
  return(d->boardlist);
}

neo_portlist *neo_device_get_portlist(neo_device *d)
{
  return(d->portlist);
}

int neo_device_is_probed(neo_device *d)
{
  return(d->constset);
}

void neo_device_free(neo_device *d)
{
  if (d==NULL) return;
  
  if (d->boardlist) neo_boardlist_free_all(d->boardlist);
  if (d->portlist) neo_portlist_free_all(d->portlist);
  if (d->name) free(d->name);
  if (d->devname) free(d->devname);
  neo_layout_free(&(d->layout));
}

void neo_device_debugprint(neo_device *d)
{
  printf("Device name: %s\n", neo_device_get_name(d));
  printf("ShortType: %s\n", neo_device_get_devname(d));
  printf("LongType: %s\n", neo_device_get_fullname(d));
  printf("HasBoards: %i\n", neo_device_has_boards(d));
}

int neo_device_command_port_sed(neo_device *d, neo_location *loc, int command)
{
  int ret;

  if (!d->constset) {
    neo_error=NEO_ERR_CONTACT;
    return(-1);
  }
  ret=(* d->func_port_sed)(loc, command);
  return(ret);
}

int neo_device_command_port_search(neo_device *d, neo_location *loc)
{
  int ret;

  if (!d->constset) {
    neo_error=NEO_ERR_CONTACT;
    return(-1);
  }
  ret=(* d->func_port_search)(loc);
  return(ret);
}

int neo_device_command_locate(neo_device *d, neo_searchable *s, neo_locationlist *l, int uplink)
{
  if (!d->func_locate) {
    neo_error=NEO_ERR_CONTACT;
    return(-1);
  }
  return((* d->func_locate)(d, s, l, uplink));
}

int neo_device_command_simplelocate(neo_device *d, neo_searchable *s, neo_locationlist *l)
{
  if (!d->func_locate) {
    neo_error=NEO_ERR_CONTACT;
    return(-1);
  }
  return((* d->func_locate_simple)(d, s, l));
}

int neo_device_command_summary(neo_device *d, neo_location *loc, neo_list *ret)
{
  if (!d->func_summary) {
    neo_error=NEO_ERR_CONTACT;
    return(-1);
  }

  return((* d->func_summary)(d, loc, ret));
}

int neo_device_command_reset(neo_device *d)
{
  return((* d->func_reset)(d));
}

int neo_device_command_load(neo_device *d, neo_location *loc, int type, int direction, char *host, char *file)
{
  if (!d->func_load) {
    neo_error=NEO_ERR_CONTACT;
    return(0);
  }

  return((* d->func_load)(loc, type, direction, host, file));
}

int neo_device_command_stats(neo_device *d, neo_location *loc, neo_list *list)
{
  if (!d->func_stats) {
    neo_error=NEO_ERR_CONTACT;
    return(-1);
  }
  return((* d->func_stats)(loc, list));
}

int neo_device_command_info(neo_device *d, int type)
{
  if (!d->func_info) {
    neo_error=NEO_ERR_CONTACT;
    return(-1);
  }
  (* d->func_info)(d, type);
  return(0);
}

/* caller must free the return */
char *neo_device_get_port_descr(neo_device *d, neo_location *loc)
{
  if (!d->func_get_port_descr) {
    neo_error=NEO_ERR_CONTACT;
    return(strdup("?"));
  }
  return((* d->func_get_port_descr)(d, loc));
}

int neo_device_get_ifindex_from_location(neo_device *d, neo_location *loc)
{
  neo_boardport bp;

  neo_location_extract_boardport(loc, &bp);
  return(neo_device_get_ifindex_from_boardport(d, &bp));
}

int neo_device_get_ifindex_from_boardport(neo_device *d, neo_boardport *bp)
{
  if (!d->func_get_ifindex_from_boardport) {
    neo_error=NEO_ERR_CONTACT;
    return(-1);
  }
  return((* d->func_get_ifindex_from_boardport)(d, bp));
}

int neo_device_set_boardport_from_ifindex(neo_device *d, neo_boardport *bp, int index)
{
  if (!d->func_set_boardport_from_ifindex) {
    neo_error=NEO_ERR_CONTACT;
    return(-1);
  }
  return((* d->func_set_boardport_from_ifindex)(d, bp, index));
}

void neo_device_print_capabilities(neo_device *d)
{
  printf("%s (%s):\n", neo_device_get_name(d), neo_device_get_devname(d));


  printf("  Locate           : ");
  if (d->func_locate_simple==&neo_command_locate_simple_generic) {
    printf("Available (Generic)\n");
  } else if (d->func_locate_simple==&neo_command_locate_simple_null) {
    printf("Unavailable\n");
  } else if (d->func_locate_simple==NULL) {
    printf("Unknown\n");
  } else {
    printf("Available\n");
  }
  
  printf("  Port Stat/Ena/Dis: ");
  if (d->func_port_sed==&neo_command_port_sed_generic) {
    printf("Available (Generic)\n");
  } else if (d->func_port_sed==&neo_command_port_sed_null) {
    printf("Unavailable\n");
  } else if (d->func_port_sed==NULL) {
    printf("Unknown\n");
  } else {
    printf("Available\n");
  }

  printf("  Port Search      : ");
  if (d->func_port_search==&neo_command_port_search_generic) {
    printf("Available (Generic)\n");
  } else if (d->func_port_search==&neo_command_port_search_null) {
    printf("Unavailable\n");
  } else if (d->func_port_search==NULL) {
    printf("Unknown\n");
  } else {
    printf("Available\n");
  }

  printf("  Summary          : ");
  if (d->func_summary==&neo_command_device_summary_generic) {
    printf("Available (Generic)\n");
  } else if (d->func_summary==&neo_command_device_summary_null) {
    printf("Unavailable\n");
  } else if (d->func_summary==NULL) {
    printf("Unknown\n");
  } else {
    printf("Available\n");
  }
    
  printf("  Stats            : ");
  if (d->func_stats==&neo_command_stats_generic) {
    printf("Available (Generic)\n");
  } else if (d->func_stats==&neo_command_stats_null) {
    printf("Unavailable\n");
  } else if (d->func_stats==NULL) {
    printf("Unknown\n");
  } else {
    printf("Available\n");
  }

  
  printf("  Reset            : ");
  if (d->func_reset==&neo_command_device_reset_null) {
    printf("Unavailable\n");
  } else if (d->func_reset==NULL) {
    printf("Unknown\n");
  } else {
    printf("Available\n");
  }

  
  printf("  Load             : ");
  if (d->func_load==&neo_command_device_load_null) {
    printf("Unavailable\n");
  } else if (d->func_load==NULL) {
    printf("Unknown\n");
  } else {
    printf("Available\n");
  }

  /*
  d->func_info=&neo_command_device_info_null;
  d->func_get_ifindex_from_boardport=&neo_helper_get_ifindex_from_boardport_generic;
  d->func_set_boardport_from_ifindex=&neo_helper_set_boardport_from_ifindex_generic;
  d->func_get_port_descr=&neo_helper_device_get_port_descr_generic;
  d->func_get_board_descr=&neo_helper_device_get_board_descr_generic;
  d->func_set_layout=&neo_helper_device_set_layout_generic;
  */


}

int _neo_device_setconsts_from_oid(neo_device *d, char *oid)
{
  int type;

  
  type=GENERIC_DEVICE;
  if (!oid) {
    /* set an error */
    return(-1);
  }
  
  if (!strcmp(oid, ".1.3.6.1.4.1.298.2.2.3")) {
    type=HUB_ASN_2072;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.298.2.2.1")) {
    type=HUB_ASN_1012;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.298.2.2.2")) {
    type=HUB_ASN_1012_BDG;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.298.2.2.7")) {
    type=HUB_ASN_2024;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.298.2.2.15")) {
    type=HUB_ASN_5324;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.298.2.2.16")) {
    type=HUB_ASN_6216;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.52")) {
    type=HUB_CTN_MRXI;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.52.3.9.3.4.84")) {
    type=HUB_CTN_2200;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.52.3.9.3.4.80")) {
    type=HUB_CTN_2200;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.1916.2.19")) {
    type=HUB_EXT_SUM1I;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.1916.2.7")) {
    type=HUB_EXT_SUM24;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.1916.2.6")) {
    type=HUB_EXT_SUM48;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.1916.2.17")) {
    type=HUB_EXT_ALP;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.1916.2.11")) {
    type=HUB_EXT_BD;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.11.2.3.7.11.19")) {
    type=HUB_HP_2524;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.5.42")) {
    type=HUB_CIS_C2948;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.1.325")) {
    type=HUB_CIS_C2950;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.5.46")) {
    type=HUB_CIS_C4000;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.1.448")) {
    type=HUB_CIS_C4000; /* Weird */
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.5.7")) {
    type=HUB_CIS_C5000;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.5.29")) {
    type=HUB_CIS_C5002;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.5.34")) {
    type=HUB_CIS_C5500;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.5.17")) {
    type=HUB_CIS_C5500;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.1.17")) {
    type=RTR_CIS_2500_IOS;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.1.185")) {
    type=RTR_CIS_2600_IOS;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.1.278")) {
    type=RTR_CIS_3500_IOS;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.1.14")) {
    type=RTR_CIS_4500_IOS;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.1.283")) {
    type=RTR_CIS_6509_IOS;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.1.46")) {
    type=RTR_CIS_7500_IOS;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.9.1.203")) {
    type=RTR_CIS_8500_IOS;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.1751.1.4.1")) {
    type=HUB_LUC_AP1000;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.762.2")) {
    type=HUB_LUC_AP1000; /* really a wavepoint II */
  } else if (!strcmp(oid, ".1.3.6.1.4.1.1751.1.4.6")) {
    type=HUB_LUC_AP2000;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.11898.2.4.6")) {
    type=HUB_AVA_AP3;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.551.2.1.77")) {
    type=HUB_CIS_AP;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.318.1.3.5.4")) {
    type=UPS_APC;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.318.1.3.3.2")) {
    type=UPS_APC;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.318.1.3.2.11")) {
    type=UPS_APC;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.318.1.3.2.10")) {
    type=UPS_APC;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.705.1.4")) {
    type=UPS_MGE_COMET;
  } else if (!strcmp(oid, ".1.3.6.1.4.1.11.2.3.9.1")) {
    type=PRINTER_HP_GENERIC;
  }

  d->type=type;
  d->constset=1;

  /* set the defaults, then only override as necessary */
  d->hasboards=0;
  d->func_port_sed=&neo_command_port_sed_generic;
  d->func_port_search=&neo_command_port_search_generic;
  d->func_locate=&neo_command_locate_generic;
  d->func_locate_simple=&neo_command_locate_simple_generic;
  d->func_stats=&neo_command_stats_generic;
  d->func_summary=&neo_command_device_summary_generic;
  d->func_load=&neo_command_device_load_null;
  d->func_reset=&neo_command_device_reset_null;
  d->func_info=&neo_command_device_info_null;
  d->func_get_ifindex_from_boardport=&neo_helper_get_ifindex_from_boardport_generic;
  d->func_set_boardport_from_ifindex=&neo_helper_set_boardport_from_ifindex_generic;
  d->func_get_port_descr=&neo_helper_device_get_port_descr_generic;
  d->func_get_board_descr=&neo_helper_device_get_board_descr_generic;
  d->func_set_layout=&neo_helper_device_set_layout_generic;

  if (type==HUB_CTN_2200) {
    d->devname=strdup("C2200");
    d->fullname=strdup("Cabletron 2200 Switch");
  } else if (type==HUB_ASN_5324) {
    d->devname=strdup("5324");
    d->fullname=strdup("Asante 5324 IntraSwitch");
  } else if (type==HUB_ASN_2072) {
    d->devname=strdup("2072");
    d->fullname=strdup("Asante 2072 Repeater");
    d->hasboards=1;
    d->func_port_sed=&neo_command_port_sed_2072;
    d->func_port_search=&neo_command_port_search_2072;
    d->func_locate_simple=&neo_command_locate_simple_2072;
    d->func_stats=&neo_command_stats_2072;
    d->func_summary=&neo_command_device_summary_2072;
    d->func_reset=&neo_command_device_reset_2072;
  } else if (type==HUB_ASN_1012) {
    d->devname=strdup("1012");
    d->fullname=strdup("Asante 1012 Repeater");
    d->hasboards=1;
    d->func_port_sed=&neo_command_port_sed_2072;
    d->func_port_search=&neo_command_port_search_2072;
    d->func_locate_simple=&neo_command_locate_simple_2072;
    d->func_stats=&neo_command_stats_1012;
    d->func_summary=&neo_command_device_summary_1012;
    d->func_reset=&neo_command_device_reset_2072;          /* TEST THIS */
  } else if (type==HUB_ASN_1012_BDG) {
    d->devname=strdup("1012B");
    d->fullname=strdup("Asante 1012 Bridge");
    d->hasboards=1;
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_port_search=&neo_command_port_search_2072;
    d->func_locate_simple=&neo_command_locate_simple_2072;
    d->func_stats=&neo_command_stats_null;
    d->func_summary=&neo_command_device_summary_null;
  } else if (type==HUB_ASN_2024) {
    d->devname=strdup("NetStacker");
    d->fullname=strdup("Asante NetStacker Repeater");
    d->hasboards=1;
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_port_search=&neo_command_port_search_null;
    d->func_locate=&neo_command_locate_null;
    d->func_locate_simple=&neo_command_locate_simple_null;
    d->func_stats=&neo_command_stats_null;
    d->func_summary=&neo_command_device_summary_null;
  } else if (type==HUB_CTN_MRXI) {
    d->devname=strdup("MRXI");
    d->fullname=strdup("Cabletron MRXI");
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_port_search=&neo_command_port_search_null;
    d->func_locate=&neo_command_locate_null;
    d->func_locate_simple=&neo_command_locate_simple_null;
    d->func_stats=&neo_command_stats_null;
    d->func_summary=&neo_command_device_summary_null;
  } else if (type==HUB_CIS_C2948) {
    d->devname=strdup("Cat2948");
    d->fullname=strdup("Catalyst 2948 Series Switch");
    d->hasboards=1;
    d->func_locate_simple=&neo_command_locate_simple_cat2948;
    d->func_stats=&neo_command_stats_cat5000;            /** TEST MORE THOROUGHLY **/
    d->func_summary=&neo_command_device_summary_cat4000; /** THIS IS UNTESTED     **/
    d->func_info=&neo_command_device_info_cat5000;
    d->func_get_ifindex_from_boardport=&neo_helper_get_ifindex_from_boardport_cat5000;
    d->func_set_boardport_from_ifindex=&neo_helper_set_boardport_from_ifindex_cat5000;
  } else if (type==HUB_CIS_C2950) {
    d->devname=strdup("Cat2950");
    d->fullname=strdup("Catalyst 2950 Series Switch");
    d->hasboards=1;
    d->func_locate_simple=&neo_command_locate_simple_cat2948;  /* TEST THIS */
    d->func_stats=&neo_command_stats_cat5000;            /** TEST MORE THOROUGHLY **/
    d->func_info=&neo_command_device_info_cat5000;
    /* need to set layout */
  } else if (type==HUB_CIS_C4000) {
    d->devname=strdup("Cat4000");
    d->fullname=strdup("Catalyst 4000 Series Switch");
    d->hasboards=1;
    d->func_locate_simple=&neo_command_locate_simple_cat2948;
    d->func_stats=&neo_command_stats_cat5000;              /* UNTESTED */
    d->func_summary=&neo_command_device_summary_cat4000;
    d->func_info=&neo_command_device_info_cat5000;
    d->func_get_ifindex_from_boardport=&neo_helper_get_ifindex_from_boardport_cat5000;
    d->func_set_boardport_from_ifindex=&neo_helper_set_boardport_from_ifindex_cat5000;
    /* need to set layout */
  } else if (type==HUB_CIS_C5000) {
    d->devname=strdup("Cat5000");
    d->fullname=strdup("Catalyst 5000 Series Switch");
    d->hasboards=1;
    d->func_stats=&neo_command_stats_cat5000;
    d->func_summary=&neo_command_device_summary_cat5000;
    d->func_load=&neo_command_device_load_cat5000;
    d->func_info=&neo_command_device_info_cat5000;
    d->func_get_ifindex_from_boardport=&neo_helper_get_ifindex_from_boardport_cat5000;
    d->func_set_boardport_from_ifindex=&neo_helper_set_boardport_from_ifindex_cat5000;
    d->func_set_layout=&neo_helper_device_set_layout_cat5000;
  } else if (type==HUB_CIS_C5002) {
    d->devname=strdup("Cat5002");
    d->fullname=strdup("Catalyst 5002 Switch");
    d->hasboards=1;
    d->func_stats=&neo_command_stats_cat5000;
    d->func_summary=&neo_command_device_summary_cat5000;
    d->func_load=&neo_command_device_load_cat5000;
    d->func_info=&neo_command_device_info_cat5000;
    d->func_get_ifindex_from_boardport=&neo_helper_get_ifindex_from_boardport_cat5000;
    d->func_set_boardport_from_ifindex=&neo_helper_set_boardport_from_ifindex_cat5000;
    /* need to set layout */
  } else if (type==HUB_CIS_C5500) {
    d->devname=strdup("Cat5500");
    d->fullname=strdup("Catalyst 5500 Series Switch");
    d->hasboards=1;
    d->func_stats=&neo_command_stats_cat5000;
    d->func_summary=&neo_command_device_summary_cat5000;
    d->func_load=&neo_command_device_load_cat5000;
    d->func_info=&neo_command_device_info_cat5000;
    d->func_get_ifindex_from_boardport=&neo_helper_get_ifindex_from_boardport_cat5000;
    d->func_set_boardport_from_ifindex=&neo_helper_set_boardport_from_ifindex_cat5000;
    /* need to set layout */
  } else if (type==RTR_CIS_7500_IOS || type==RTR_CIS_2600_IOS ||
	     type==RTR_CIS_3500_IOS || type==RTR_CIS_2500_IOS ||
	     type==RTR_CIS_4500_IOS) {
    d->devname=strdup("Cisco Router");
    d->fullname=strdup("Cisco Router");
    d->func_port_sed=&neo_command_port_sed_ciscorouter;
    d->func_port_search=&neo_command_port_search_null;
    d->func_locate=&neo_command_locate_null;
    d->func_stats=&neo_command_stats_ciscorouter;
    d->func_summary=&neo_command_device_summary_ciscorouter;
    d->func_info=&neo_command_device_info_ciscorouter_old;
  } else if (type==RTR_CIS_8500_IOS || type==RTR_CIS_6509_IOS) {
    d->devname=strdup("Cisco Router");
    d->fullname=strdup("Cisco Router");
    d->func_port_sed=&neo_command_port_sed_ciscorouter;
    d->func_port_search=&neo_command_port_search_null;
    d->func_locate=&neo_command_locate_null;
    d->func_stats=&neo_command_stats_ciscorouter;
    d->func_summary=&neo_command_device_summary_ciscorouter;
    d->func_info=&neo_command_device_info_ciscorouter_new;
  } else if (type==HUB_BST_450F) { /* test */
    d->devname=strdup("BayStack 450");
    d->fullname=strdup("BayStack 450");
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_port_search=&neo_command_port_search_null;
    d->func_locate=&neo_command_locate_null;
    d->func_stats=&neo_command_stats_null;
    d->func_summary=&neo_command_device_summary_null;
  } else if (type==HUB_LUC_AP) {
    d->devname=strdup("Lucent Wireless");
    d->fullname=strdup("Lucent Wireless Access Point");
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_port_search=&neo_command_port_search_lucentwav;
    d->func_stats=&neo_command_stats_null;
    d->func_summary=&neo_command_device_summary_null;
    d->func_reset=&neo_command_device_reset_lucentwav;
  } else if (type==HUB_LUC_AP1000) {
    d->devname=strdup("Lucent Wireless 1000");
    d->fullname=strdup("Lucent Wireless Access Point 1000");
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_port_search=&neo_command_port_search_lucentwav;
    d->func_reset=&neo_command_device_reset_lucentwav;
  } else if (type==HUB_LUC_AP2000) {
    d->devname=strdup("Lucent Wireless 2000");
    d->fullname=strdup("Lucent Wireless Access Point 2000");
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_port_search=&neo_command_port_search_lucentwav;
    d->func_stats=&neo_command_stats_null;
    d->func_reset=&neo_command_device_reset_lucentwav;
  } else if (type==HUB_AVA_AP3) {
    d->devname=strdup("Avaya AP 3");
    d->fullname=strdup("Avaya AP 3");
    d->func_reset=&neo_command_device_reset_null;
  } else if (type==HUB_CIS_AP) {
    d->devname=strdup("Cisco Wireless");
    d->fullname=strdup("Cisco Aironet Acesss Point");
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_stats=&neo_command_stats_null;
    d->func_summary=&neo_command_device_summary_null;
  } else if (type==HUB_EXT_SUM1I) {
    d->devname=strdup("Summit 1i");
    d->fullname=strdup("Extreme Summit 1i");
    d->func_stats=&neo_command_stats_extsum1;
    d->func_summary=&neo_command_device_summary_extsum1;
    d->func_info=&neo_command_device_info_extalp;
  } else if (type==HUB_EXT_SUM24) {
    d->devname=strdup("Summit 24");
    d->fullname=strdup("Extreme Summit 24");
    d->func_stats=&neo_command_stats_extsum24;
    d->func_summary=&neo_command_device_summary_extsum24;
    d->func_info=&neo_command_device_info_extalp;
  } else if (type==HUB_EXT_SUM48) {
    d->devname=strdup("Summit 48");
    d->fullname=strdup("Extreme Summit 48");
    d->func_stats=&neo_command_stats_extsum48;
    d->func_summary=&neo_command_device_summary_extsum48;
    d->func_info=&neo_command_device_info_extalp;
  } else if (type==HUB_EXT_ALP) {
    d->devname=strdup("Apline");
    d->fullname=strdup("Extreme Alpine");
    d->hasboards=1;
    d->func_stats=&neo_command_stats_extalp;
    d->func_summary=&neo_command_device_summary_extalp;
    d->func_info=&neo_command_device_info_extalp;
    d->func_get_ifindex_from_boardport=&neo_helper_get_ifindex_from_boardport_extalp;
    d->func_set_boardport_from_ifindex=&neo_helper_set_boardport_from_ifindex_extalp;
  } else if (type==HUB_EXT_BD) {
    d->devname=strdup("Black Diamond");
    d->fullname=strdup("Extreme Black Diamond");
    d->hasboards=1;
    d->func_stats=&neo_command_stats_extalp;
    d->func_summary=&neo_command_device_summary_extalp;
    d->func_info=&neo_command_device_info_extalp;
  } else if (type==HUB_HP_2524) {
    d->devname=strdup("HP 2524");
    d->fullname=strdup("HP 2524");
    d->func_stats=&neo_command_stats_hp2524;
    d->func_summary=&neo_command_device_summary_hp2524;
  } else if (type==UPS_APC) {
    d->devname=strdup("APC UPS");
    d->fullname=strdup("APC UPS");
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_port_search=&neo_command_port_search_null;
    d->func_locate=&neo_command_locate_null;
    d->func_stats=&neo_command_stats_null;
    d->func_summary=&neo_command_device_summary_null;
    d->func_info=&neo_command_device_info_apcups;
  } else if (type==UPS_MGE_COMET) {
    d->devname=strdup("MGE UPS");
    d->fullname=strdup("MGE COMET UPS");
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_port_search=&neo_command_port_search_null;
    d->func_locate=&neo_command_locate_null;
    d->func_stats=&neo_command_stats_null;
    d->func_summary=&neo_command_device_summary_null;
  } else if (type==PRINTER_HP_GENERIC) {
    d->devname=strdup("HP Printer");
    d->fullname=strdup("HP Printer");
    d->func_port_sed=&neo_command_port_sed_null;
    d->func_port_search=&neo_command_port_search_null;
    d->func_locate=&neo_command_locate_null;
    d->func_stats=&neo_command_stats_null;
    d->func_summary=&neo_command_device_summary_null;
    d->func_reset=&neo_command_device_reset_hpprinter;
    d->func_info=&neo_command_device_info_hpprinter;
  } else {
    d->type=GENERIC_DEVICE;
    d->devname=strdup("Generic");
    d->fullname=strdup("Generic Device");
  }
  return(0);
}
