#include "neo.h"
#include <stdlib.h>

/* locationlist */
int neo_locationlist_create(neo_locationlist *list)
{
  return(neo_list_create(list));
}

int neo_locationlist_append_element(neo_locationlist *list, neo_location *loc)
{
  return(neo_list_append_element(list, loc));
}

neo_location *neo_locationlist_get_element(neo_locationlist *list, int index)
{
  return(neo_list_get_element(list, index));
}

int neo_locationlist_get_size(neo_locationlist *list)
{
  return(neo_list_get_size(list));
}

int neo_locationlist_remove_element(neo_locationlist *l, int n)
{
  return(neo_list_remove_element(l, n));
}

void neo_locationlist_copy(neo_locationlist *src, neo_locationlist *dst)
{
  return(neo_list_copy(src, dst));
}

/* Create the location list c, that has locations in a that are not
 * also in b.  Note! Pointers for devices etc. will be into a or b,
 * independant copies will not be made.  'c' should already be
 * initialized.
 */
void neo_locationlist_prune(neo_locationlist *c, neo_locationlist *a, neo_locationlist *b)
{
  int i, j, x, y, index_offset;
  neo_location *loca, *locb;

  j=neo_locationlist_get_size(a);
  y=neo_locationlist_get_size(b);
  /* add everything from a to c */
  for (i=0; i<j; i++) {
    loca=neo_locationlist_get_element(a, i);
    neo_locationlist_append_element(c, loca);
  }

  /* remove anything from c that's in a and b */
  index_offset=0;
  for (i=0; i<j; i++) {
    loca=neo_locationlist_get_element(a, i);
    for (x=0; x<y; x++) {
      locb=neo_locationlist_get_element(b, x);
      /* printf("DEBUG: Pruning %s and %s\n", neo_location_to_string(loca), neo_location_to_string(locb)); */
      if (neo_location_is_equal(loca, locb)) {
	neo_locationlist_remove_element(c, i-index_offset);
	index_offset++;
      }
    }
  }
}

/* Return 1 if the location 'loc' is present in the location list,
 * otherwise return 0.  This does NOT expand the device list to check
 * every board/port@device combination.
 */
int neo_locationlist_has_location(neo_locationlist *list, neo_location *loc)
{
  int i, j;

  j=neo_locationlist_get_size(list);
  for (i=0; i<j; i++) {
    if (neo_location_is_equal(loc, neo_locationlist_get_element(list, i))) return(1);
  }
  return(0);
}

/* Free the list and all the locations that are part
 * of it.
 */
void neo_locationlist_free_all(neo_locationlist *list)
{
  void (*func)();

  func=&neo_location_free;
  neo_list_free_all(list, func);
}

void neo_locationlist_free_simple(neo_locationlist *list)
{
  neo_list_free_simple(list);
}
