#include <stdio.h>
#include <strings.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "neo.h"

/* create a mac address from 6 base10 integers */
void neo_macaddr_create_from_ints(neo_macaddr *m, int a, int b, int c, int d, int e, int f)
{
  m->a=a;
  m->b=b;
  m->c=c;
  m->d=d;
  m->e=e;
  m->f=f;
}

/* create a mac address from a string of six dotted integers, as
 * you might find in an snmp variable. */
void neo_macaddr_create_from_dotted_ints(neo_macaddr *m, char *in)
{
  int num[6], count;
  char *ptr;
  
  count=0;
  ptr=in;
  while(1) {
    num[count]=atoi(ptr);
    ptr=strchr(ptr, '.');
    if (!ptr) break;
    ptr++;
    count++;
  }
  
  neo_macaddr_create_from_ints(m, num[0], num[1], num[2], num[3], num[4], num[5]);
}


/* create a macaddress from a string.  The string can be in the format:
 *     xxxx.xxxx.xxxx
 *     xx:xx:xx:xx:xx:xx
 *     xx xx xx xx xx xx
 *     xx-xx-xx-xx-xx-xx
 * where 'x' is a hex digit.
 * Return 0 on success, -1 on error.
 *
 * FUTURE: clean this up and make it accept single digit components.
 */
int neo_macaddr_create_from_string(neo_macaddr *m, char *in)
{
  char *ptr, *endptr, buff[5];
  int i, j, num[10], len;

  /* for convenience in SNMP responses */
  ptr=strstr(in, "Hex:");
  if (ptr) {
    ptr+=strlen("Hex:");
  } else {
    ptr=in;
  }

  /* remove up to 5 leading spaces */
  for (i=0; i<5; i++) {
    if (ptr[0]==' ') ptr++;
  }

  len=strlen(ptr);
  endptr=ptr+len-1;

  if (len<12) {
    neo_error=NEO_ERR_BADFORMAT;
    return(-1);
  }

  /* format restrictions */
  if (! ((ptr[4]=='.' && ptr[9]=='.') ||
	 (ptr[2]==':' && ptr[5]==':' && ptr[8]==':' && ptr[11]==':' && ptr[14]==':') ||
	 (ptr[2]==' ' && ptr[5]==' ' && ptr[8]==' ' && ptr[11]==' ' && ptr[14]==' ') ||
	 (ptr[2]=='-' && ptr[5]=='-' && ptr[8]=='-' && ptr[11]=='-' && ptr[14]=='-'))) {
    neo_error=NEO_ERR_BADFORMAT;
    return(-1);
  }

  /* need to be very specific about xx-xx-xx-xx-xx-xx so it's not
   * confused with a hostname */
  if (ptr[2]=='-') {
    if (! (isxdigit((int) ptr[0]) && isxdigit((int) ptr[1]) &&
	   isxdigit((int) ptr[3]) && isxdigit((int) ptr[4]) &&
	   isxdigit((int) ptr[6]) && isxdigit((int) ptr[7]) &&
	   isxdigit((int) ptr[9]) && isxdigit((int) ptr[10]) &&
	   isxdigit((int) ptr[12]) && isxdigit((int) ptr[13]) &&
	   isxdigit((int) ptr[15]) && isxdigit((int) ptr[16]))) {
      neo_error=NEO_ERR_BADFORMAT;
      return(-1);
    }
  }

  /* need to be very specific about xxxx.xxxx.xxxx so it's not
   * confused with a hostname */
  if (ptr[4]=='.') {
    if (! (isxdigit((int) ptr[0]) && isxdigit((int) ptr[1]) && isxdigit((int) ptr[2]) && isxdigit((int) ptr[3]) &&
	   isxdigit((int) ptr[5]) && isxdigit((int) ptr[6]) && isxdigit((int) ptr[7]) && isxdigit((int) ptr[8]) &&
	   isxdigit((int) ptr[10]) && isxdigit((int) ptr[11]) && isxdigit((int) ptr[12]) && isxdigit((int) ptr[13]))) {
      neo_error=NEO_ERR_BADFORMAT;
      return(-1);
    }
  }
  
  num[0]=num[1]=num[2]=num[3]=num[4]=num[5]=-1;
  for (i=0; i<6; i++) {
    if (ptr>endptr-1) {
      neo_error=NEO_ERR_BADFORMAT;
      return(-1);
    }
    buff[0]=ptr[0];
    buff[1]=ptr[1];
    buff[2]='\0';
    sscanf(buff, "%x", &j);
    num[i]=j;

    ptr+=2;
    if (i!=5 && !isxdigit((int) ptr[0])) ptr++;
  }
  
  /* check if any weren't set */
  for (i=0; i<6; i++) {
    if (num[i]==-1) {
      neo_error=NEO_ERR_BADFORMAT;
      return(-1);
    }
  }

  neo_macaddr_create_from_ints(m, num[0], num[1], num[2], num[3], num[4], num[5]);
  return(0);
}


/* Print the mac address in a string representation into 'buff'.  Use
 * the format specified by the global variable 'macmode'. */
void neo_macaddr_to_string(neo_macaddr *m, char *buff)
{
  if (neo_global_get_macmode(g)==NEO_GLOBAL_MACMODE_STANDARD) {
    sprintf(buff, "%2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X", m->a, m->b, m->c, m->d, m->e, m->f);
  } else if (neo_global_get_macmode(g)==NEO_GLOBAL_MACMODE_CISCO) {
    sprintf(buff, "%2.2X%2.2X.%2.2X%2.2X.%2.2X%2.2X", m->a, m->b, m->c, m->d, m->e, m->f);
  } else if (neo_global_get_macmode(g)==NEO_GLOBAL_MACMODE_DASH) {
    sprintf(buff, "%2.2X-%2.2X-%2.2X-%2.2X-%2.2X-%2.2X", m->a, m->b, m->c, m->d, m->e, m->f);
  } else {
    sprintf(buff, "%2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X", m->a, m->b, m->c, m->d, m->e, m->f);
  }
}


/* Print a string version of the mac address 'm' in 6 dotted ints
 * into 'buff' */
void neo_macaddr_to_dotted_ints_string(neo_macaddr *m, char *buff)
{
  sprintf(buff, "%i.%i.%i.%i.%i.%i", m->a, m->b, m->c, m->d, m->e, m->f);
}


/* if mac address 'a' and mac address 'b' are equivalent return 1,
 * otherwise return 0 */
int neo_macaddr_is_equal(neo_macaddr *a, neo_macaddr *b)
{
  if (a->a != b->a) return(0);
  if (a->b != b->b) return(0);
  if (a->c != b->c) return(0);
  if (a->d != b->d) return(0);
  if (a->e != b->e) return(0);
  if (a->f != b->f) return(0);
  return(1);
}
